@@ -24,11 +24,13 @@ using std::endl;
 
 extern "C" {
 #include "analyze-linkage.h"
+#include "build-disjuncts.h"
 #include "dict-file/read-dict.h"
 #include "extract-links.h"
 #include "linkage.h"
 #include "post-process.h"
 #include "preparation.h"
+#include "score.h"
 #include "utilities.h"
 }
 
@@ -36,7 +38,7 @@ extern "C" {
 #ifdef _DEBUG
 #define DEBUG_print(x) (cout << x << endl)
 #else
-#define DEBUG_print(x) 
+#define DEBUG_print(x)
 #endif
 
 /*-------------------------------------------------------------------------*
@@ -251,7 +253,8 @@ void SATEncoder::build_word_tags()
     bool leading_left = true;
     std::vector<int> eps_right, eps_left;
 
-    _word_tags[w].insert_connectors(exp, dfs_position, leading_right, leading_left, eps_right, eps_left, name, true, 0);
+    _word_tags[w].insert_connectors(exp, dfs_position, leading_right,
+         leading_left, eps_right, eps_left, name, true, 0, NULL);
 
     if (join)
       free_alternatives(exp);
@@ -331,7 +334,7 @@ void SATEncoder::generate_satisfaction_for_expression(int w, int& dfs_position,
   if (e->type == CONNECTOR_type) {
     dfs_position++;
 
-    generate_satisfaction_for_connector(w, dfs_position, e->u.string, e->dir, e->multi, e->cost, var);
+    generate_satisfaction_for_connector(w, dfs_position, e, var);
 
     if (total_cost > _cost_cutoff) {
       Lit lhs = Lit(_variables->string_cost(var, e->cost));
@@ -440,7 +443,7 @@ Exp* SATEncoder::join_alternatives(int w)
   Exp* exp;
   E_list* or_list = NULL;;
   for (X_node* x = _sent->word[w].x; x != NULL; x = x->next) {
-    E_list* new_node = (E_list*)xalloc(sizeof(E_list));
+    E_list* new_node = (E_list*) xalloc(sizeof(E_list));
     new_node->e = x->exp;
     new_node->next = NULL;
     if (or_list == NULL) {
@@ -472,9 +475,12 @@ void SATEncoder::free_alternatives(Exp* exp)
 }
 
 
-void SATEncoder::generate_link_cw_ordinary_definition(size_t wi, int pi, const char* Ci,
-                                                      char dir, double cost, size_t wj)
+void SATEncoder::generate_link_cw_ordinary_definition(size_t wi, int pi,
+                                                      Exp* e, size_t wj)
 {
+  const char* Ci = e->u.string;
+  char dir = e->dir;
+  double cost = e->cost;
   Lit lhs = Lit(_variables->link_cw(wj, wi, pi, Ci));
 
   char str[MAX_VARIABLE_NAME];
@@ -492,12 +498,16 @@ void SATEncoder::generate_link_cw_ordinary_definition(size_t wi, int pi, const c
       continue;
 
     if (dir == '+') {
-      rhs.push(Lit(_variables->link_cost(wi, pi, Ci,
-                                         (*i)->word, (*i)->position, (*i)->connector->string,
+      rhs.push(Lit(_variables->link_cost(wi, pi, Ci, e,
+                                         (*i)->word, (*i)->position,
+                                         (*i)->connector->string,
+                                         (*i)->exp,
                                          cost + (*i)->cost)));
     } else if (dir == '-'){
-      rhs.push(Lit(_variables->link((*i)->word, (*i)->position, (*i)->connector->string,
-                                    wi, pi, Ci)));
+      rhs.push(Lit(_variables->link((*i)->word, (*i)->position,
+                                    (*i)->connector->string,
+                                    (*i)->exp,
+                                    wi, pi, Ci, e)));
     }
   }
 
@@ -1091,8 +1101,9 @@ void SATEncoder::power_prune()
 
         add_additional_power_pruning_conditions(clause, wl, (*lci)->word);
 
-        clause.push(~Lit(_variables->link(wl, rci->position, rci->connector->string,
-                                          (*lci)->word, (*lci)->position, (*lci)->connector->string)));
+        clause.push(~Lit(_variables->link(
+               wl, rci->position, rci->connector->string, rci->exp,
+               (*lci)->word, (*lci)->position, (*lci)->connector->string, (*lci)->exp)));
         add_clause(clause);
       }
     }
@@ -1173,11 +1184,11 @@ void SATEncoder::pp_prune()
 {
   const std::vector<int>& link_variables = _variables->link_variables();
 
-  if (_sent->dict->postprocessor == NULL)
+  if (_sent->dict->base_knowledge == NULL)
     return;
 
   pp_knowledge * knowledge;
-  knowledge = _sent->dict->postprocessor->knowledge;
+  knowledge = _sent->dict->base_knowledge;
 
   for (size_t i=0; i<knowledge->n_contains_one_rules; i++)
   {
@@ -1236,31 +1247,9 @@ Linkage SATEncoder::create_linkage()
   /* Using exalloc since this is external to the parser itself. */
   Linkage linkage = (Linkage) exalloc(sizeof(struct Linkage_s));
   memset(linkage, 0, sizeof(struct Linkage_s));
+  partial_init_linkage(linkage, _sent->length);
 
-  linkage->num_words = _sent->length;
-  linkage->word = (const char **) exalloc(linkage->num_words*sizeof(char *));
-  linkage->sent = _sent;
-  linkage->opts = _opts;
-  //  linkage->info = sent->link_info[k]; done later...
-
-  if (_sent->parse_info) {
-    Parse_info pi = _sent->parse_info;
-    // ??? there's no link-array entry for the right wall..?
-    for (size_t i=0; i < _sent->length-1; i++) {
-      free_connectors(pi->link_array[i].lc);
-      free_connectors(pi->link_array[i].rc);
-    }
-    free_parse_info(_sent->parse_info);
-    _sent->parse_info = NULL;
-  }
-  Parse_info pi = parse_info_new(_sent->length);
-  sat_extract_links(pi);
-  pi->N_words = _sent->length;
-  _sent->parse_info = pi;
-
-  extract_thin_linkage(_sent, _opts, linkage);
-
-  compute_chosen_words(_sent, linkage);
+  sat_extract_links(linkage);
 
   return linkage;
 }
@@ -1284,32 +1273,35 @@ Linkage SATEncoder::get_next_linkage()
 {
   if (l_False == _solver->solve()) return NULL;
   Linkage linkage = create_linkage();
+  Linkage lkg = NULL;
 
   std::vector<int> components;
   bool connected = connectivity_components(components);
   if (connected) {
     // num_connected_linkages++;
 
-    // Expand the lifo array.
+    // Expand the linkage array.
     int index = _sent->num_linkages_alloced;
     _sent->num_linkages_alloced++;
-    size_t nbytes = _sent->num_linkages_alloced * sizeof(Linkage_info);
-    _sent->link_info = (Linkage_info*) xrealloc(_sent->link_info,
-                       nbytes - sizeof(Linkage_info), nbytes);
-    Linkage_info* lifo = &_sent->link_info[index];
+    size_t nbytes = _sent->num_linkages_alloced * sizeof(struct Linkage_s);
+    _sent->lnkages = (Linkage) realloc(_sent->lnkages, nbytes);
 
-    // Why is linkage_post_process() never called here?
-    *lifo = analyze_thin_linkage(_sent, _opts, PP_SECOND_PASS);
-    lifo->index = index;
-    linkage->info = lifo;
+    lkg = &_sent->lnkages[index];
+    *lkg = *linkage;  /* copy en-mass */
+    exfree(linkage, sizeof(struct Linkage_s));
+    linkage = NULL;
 
-    if (0 == lifo->N_violations) {
+    // perform the post-processing
+    sane_linkage_morphism(_sent, lkg, _opts);
+    linkage_post_process(lkg, _sent->postprocessor, _opts);
+    linkage_score(lkg, _opts);
+
+    if (0 == lkg->lifo.N_violations) {
       _sent->num_valid_linkages++;
-      sane_linkage_morphism(_sent, index, _opts);
     }
 
     // sane_morphism will increment N_violations if its insane...
-    if (0 == lifo->N_violations) {
+    if (0 == lkg->lifo.N_violations) {
       cout << "Linkage PP OK" << endl;
     } else {
       cout << "Linkage PP NOT OK" << endl;
@@ -1319,10 +1311,11 @@ Linkage SATEncoder::get_next_linkage()
   } else {
     cout << "Linkage DISCONNECTED" << endl;
     generate_disconnectivity_prohibiting(components);
+    exfree(linkage, sizeof(struct Linkage_s));
   }
 
   _solver->printStats();
-  return linkage;
+  return lkg;
 }
 
 /*******************************************************************************
@@ -1344,13 +1337,18 @@ void SATEncoderConjunctionFreeSentences::determine_satisfaction(int w, char* nam
   generate_literal(Lit(_variables->string(name)));
 }
 
-void SATEncoderConjunctionFreeSentences::generate_satisfaction_for_connector(int wi, int pi, const char* Ci,
-                                                                             char dir, bool multi, double cost, char* var)
+void SATEncoderConjunctionFreeSentences::generate_satisfaction_for_connector(
+    int wi, int pi, Exp *e, char* var)
 {
+  const char* Ci = e->u.string;
+  char dir = e->dir;
+  bool multi = e->multi;
+  double cost = e->cost;
+
   Lit lhs = Lit(_variables->string_cost(var, cost));
 
 #ifdef _DEBUG
-  cout << "*** Connector: ." << wi << ". ." << pi << ". " <<  Ci << dir << endl;
+  cout << "*** Connector: ." << wi << ". ." << pi << ". " << Ci << dir << endl;
 #endif
 
   // words that can potentially match (wi, pi)
@@ -1367,7 +1365,7 @@ void SATEncoderConjunctionFreeSentences::generate_satisfaction_for_connector(int
     // connector (wi, pi) and the word wj
     _w_.push_back(wj);
 
-    generate_link_cw_ordinary_definition(wi, pi, Ci, dir, cost, wj);
+    generate_link_cw_ordinary_definition(wi, pi, e, wj);
   }
 
   vec<Lit> _link_cw_;
@@ -1493,8 +1491,9 @@ void SATEncoder::generate_linked_min_max_planarity()
   }
 }
 
-bool SATEncoderConjunctionFreeSentences::sat_extract_links(Parse_info pi)
+bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
 {
+
   int current_link = 0;
   const std::vector<int>& link_variables = _variables->link_variables();
   std::vector<int>::const_iterator i;
@@ -1506,34 +1505,63 @@ bool SATEncoderConjunctionFreeSentences::sat_extract_links(Parse_info pi)
     if (_solver->model[_variables->linked(var->left_word, var->right_word)] != l_True)
       continue;
 
-    Link& clink = pi->link_array[current_link];
+    check_link_size(lkg);
+    Link& clink = lkg->link_array[current_link];
     clink.lw = var->left_word;
     clink.rw = var->right_word;
-    //    pi->link_array[j].name = var->label;
 
     Connector* connector;
 
+    // XXX Should not alloc a new one, but use the connector from
+    // the PositionConnector class ...
     connector = connector_new();
     connector->string = var->left_connector;
+    connector->word = var->left_word; // XXX is this needed ?
     clink.lc = connector;
 
     connector = connector_new();
     connector->string = var->right_connector;
+    connector->word = var->right_word; // XXX is this needed ?
     clink.rc = connector;
 
     // Indicate the disjuncts, too.
     // This is needed so that compute_chosen_word works correctly.
-#ifdef NO_DISJUNCT_INFO_AVAILABLE_WTF
-// XXX I think LinkVar needs to be extended to hold the disjunct
-    pi->chosen_disjuncts[clink.lw] = ld;
-    pi->chosen_disjuncts[clink.rw] = rd;
-#endif
+
+// FIXME -- this is deeply and fundamentally broken. First, we should not
+// use unsplit_word below, but instead, get the string from the X_node.
+// However, the X_node was discarded previously, up above.  Next, we are
+// trying to reconstruct the actual disjunct that was used, but we aren't
+// really doing that correctly, because we never recorded the DNF that
+// was used.  Maybe .. maybe we could hack around this by looking at the
+// actual connectors that were used for each word, and combine those to
+// get the disjunct ... so its a mess.
+// Its not ever clear to me why chosen_words needs disjuncts .. really
+// all we need here are the X_node strings, and nothing else, and so ...
+// all teh exp flow through this code could be removed. Arghhh.
+    Disjunct *d;
+
+    // XXX FIXME -- the chosen disjunct has probably already been
+    // set for this word .. don't set it again.  Oh, and it should
+    // be consistent, too ...
+    d = build_disjuncts_for_exp(var->left_exp,
+                                _sent->word[var->left_word].unsplit_word,
+                                UNLIMITED_LEN);
+    _sent->word[var->left_word].d = d;
+    lkg->chosen_disjuncts[clink.lw] = d;
+
+    d = build_disjuncts_for_exp(var->right_exp,
+                                _sent->word[var->right_word].unsplit_word,
+                                 UNLIMITED_LEN);
+    _sent->word[var->right_word].d = d;
+    lkg->chosen_disjuncts[clink.rw] = d;
 
     current_link++;
   }
 
-  pi->N_links = current_link;
-  DEBUG_print("Total: ." <<  pi->N_links << "." << endl);
+  lkg->num_links = current_link;
+  compute_link_names(lkg, _sent->string_set);
+
+  DEBUG_print("Total: ." <<  lkg->num_links << "." << endl);
   return false;
 }
 
@@ -1577,18 +1605,4 @@ extern "C" void sat_sentence_delete(Sentence sent)
   SATEncoder* encoder = (SATEncoder*) sent->hook;
   if (!encoder) return;
   delete encoder;
-
-  // Don't do this parse-info-free stuff, if there's no encoder.
-  // That's because it will screw up the regular parser.
-  if (sent->parse_info) {
-    Parse_info pi = sent->parse_info;
-    // ??? there's no link-array entry for the right wall..?
-    for (size_t i=0; i < sent->length-1; i++) {
-      free_connectors(pi->link_array[i].lc);
-      free_connectors(pi->link_array[i].rc);
-    }
-    free_parse_info(sent->parse_info);
-    sent->parse_info = NULL;
-  }
-
 }
