@@ -1,6 +1,4 @@
-extern "C" {
-#include <link-grammar/link-includes.h>
-}
+#include "link-includes.h"
 
 extern "C" int sat_encode(Sentence sent, Parse_Options  opts);
 extern "C" Linkage sat_create_linkage(int k, Sentence sent, Parse_Options  opts);
@@ -56,12 +54,13 @@ protected:
   virtual void determine_satisfaction(int w, char* name) = 0;
 
   // Generates satisfaction condition for the connector (wi, pi)
-  virtual void generate_satisfaction_for_connector(int wi, int pi, const char* Ci,
-                                                   char dir, bool multi, double cost, char* var) = 0;
+  virtual void generate_satisfaction_for_connector(int wi, int pi, 
+                                                   Exp* e,
+                                                   char* var) = 0;
 
   // Definition of link_cw((wi, pi), wj) variables when wj is an ordinary word
-  void generate_link_cw_ordinary_definition(size_t wi, int pi, const char* Ci, char dir,
-                                            double cost, size_t wj);
+  void generate_link_cw_ordinary_definition(size_t wi, int pi,
+                                            Exp* e, size_t wj);
 
   // Generates order constraints for the elements of a conjunction.
   void generate_conjunct_order_constraints(int w, Exp *e1, Exp* e2, int& dfs_position);
@@ -182,7 +181,7 @@ protected:
   // pruned if their cost exceeds this value. Cost cutoff is performed
   // during satisfaction condition generating.
   // XXX FIXME, this should be taken from parse options.
-  static const double _cost_cutoff = 2.7;
+  static constexpr double _cost_cutoff = 2.7;
 
   /**
    *   Creating clauses and passing them to the MiniSAT solver
@@ -264,7 +263,7 @@ protected:
    */
 
   // Convert propositional model to a parse info structure
-  virtual bool sat_extract_links(Parse_info pi) = 0;
+  virtual bool sat_extract_links(Linkage) = 0;
 
   // Create linkage from a propositional model
   Linkage create_linkage();
@@ -301,12 +300,13 @@ public:
 
   virtual void handle_null_expression(int w);
   virtual void determine_satisfaction(int w, char* name);
-  virtual void generate_satisfaction_for_connector(int wi, int pi, const char* Ci,
-                                                   char dir, bool multi, double cost, char* var);
+  virtual void generate_satisfaction_for_connector(int wi, int pi,
+                                                   Exp* e,
+                                                   char* var);
 
 
   virtual void generate_linked_definitions();
-  virtual bool sat_extract_links(Parse_info pi);
+  virtual bool sat_extract_links(Linkage);
 
   virtual void generate_encoding_specific_clauses();
 };
